double sigmcq(int iSV,double Vfac,double X,double Y,double ENU,double POL,double *p)
{
//
//     CONVERTED 25-JUL-90   BILL SELIGMAN
//
//     CALCULATES CROSS SECTION/ENU(UP TO OBVIOUS NORMALIZATION)
//       USING PARTICULAR FUNCTIONAL FORMS
//
//     PARAMETERS REQUIRING EXPLANATION
//       SFAC IS FRACTION OF SU3 SYMMETRIC SEA FOR STRANGE PART
//        (SFAC=0 TURNS OFF STRANGE SEA CORRECTION)
//       D-QUARKS FALL FASTER THAN U-QUARKS BY (1.-X)
//       VFAC IS NEUTRON EXCESS FRACTION = (N-Z)/A
//         (VFAC=0 TURNS OFF N/P CORRECTION)
//   FOR TWOMU=.T. DOES DV,DS OR SS FOR ICHARM = 1,2,3 -- WHS 12/28/87
//
//     THIS IS A NEW VERSION COMPLETELY REWRITTEN BY : COSTAS
//     IT HAS THE CORRECT DIMUON CROSSECTION
//     IT HAS THE SLOW RESCALING IMPLEMENTED IN THE SINGLE MUON
//     CROSSECTION
//     IT TAKES AS INPUT THE E701-616 STRUCTURE FUNCTIONS FROM
//     ED OLTMANS PROGRAMS.
//
//
//                            COSTAS FOUDAS  26-JUL-88
//
//    *****  PUT IN PROPAGATOR - SINGLE MUON ONLY
//                            LEUNG          APR-06-90
//
//    REMOVE CONSTRAINT ON R. WGS - 2-MAR-92.
//
//    If the cross-section falls in an unphysical region (such
//    as Emu < mass(mu), return zero.   13-Nov-95 WGS
//
//     Low Q2 extrapolation using GRV94 LO: modelmod.f , U.K. Yang
//
//
  //
  // decode parameters
  //
                                                                                
  double A      =p[iA];
  double B      =p[iB];
                                                                                
  const double CSCAB=.947;
  const double SNCAB=.053;
                                                                                
  //
  // initialize and check inputs
  //
                                                                                
  double SIGMCQ=0;
  if (X<=0||X>=1||Y<=0||Y>=1) return SIGMCQ;
                                                                                
  double Q2 = kTwoMP*ENU*X*Y;
  double EMU=ENU*(1.-Y);        // Added 14-Nov-95
  if(EMU<0.10566) return SIGMCQ;
                                                                                
  double PMU=sqrt(EMU*EMU- 0.0111639);
  double CTHETA=(2.*ENU*EMU-Q2-0.0111639)/(2.*ENU*PMU);
  if (fabs(CTHETA)>=1.0) return SIGMCQ;
                                                                                
  //
  // SINGLE MUON CROSS SECTION
  //
  // ASSUME THAT ALL SEA QUARKS HAVE THE SAME FUNCTIONAL FORM AND
  // THAT THE STRANGE QUARKS ARE DIFFERENT ONLY UP TO  KAPPA (SFAC)
  //
  // ukyang B=0.2220, A=0.4176
  // UK uses IHT==1 for the old scaling variable, IHT==2 for new..
  // I will use iSV==0 for the old, iSV==1 for the new scaling variable
                                                                                
  double XP = X + (kCmass*kCmass)/(kTwoMP*Y*ENU);
  double Xw;
  double XPw;
  if (iSV==0)
  {
    Xw =X;
    XPw=XP;
  }
  else
  {
    Xw =X *(Q2+B)/(0.5*Q2*(1.0+sqrt(1.0+pow(kTwoMP*X, 2)/Q2))+A*X );
    XPw=XP*(Q2+B)/(0.5*Q2*(1.0+sqrt(1.0+pow(kTwoMP*XP,2)/Q2))+A*XP);
  }
                                                                                
  double uv,dv,us,ds,ss;
  grv98_mod(iSV,Xw,Q2,uv,dv,us,ds,ss,p);
  double s=(us+ds)/2.0;
                                                                                
  //
  //  DO SLOW RESCALING IN THE SINGLE MUON CROSSECTION IF XP AND Y ARE
  //  IN THE KINEMATIC REGION FOR PRODUCING CHARM.
  //
                                                                                
  double uvc=0;
  double dvc=0;
  double usc=0;
  double dsc=0;
  double sc =0;
  double ssc=0;
  double HETA=(kCmass*kCmass)/(kTwoMP*ENU);
  if(XP<=1.0&&XP>=HETA&&Y>=HETA/XP)
  {
    grv98_mod(iSV,XPw,Q2,uvc,dvc,usc,dsc,ssc,p);
    uvc=(Xw/XPw)*uvc;
    dvc=(Xw/XPw)*dvc;
    sc =(Xw/XPw)*(usc+dsc)/2.0;
    ssc=(Xw/XPw)*ssc;
  }
                                                                                
  //
  //      NEUTRINO SINGLE MUONS
  //
  double Q=0;
  double QC=0;
  double QBAR=0;
  double QBARC=0;;
  if (POL>0)
  {
    Q    =((1.-Vfac)*dv +(1.+Vfac)*uv )*CSCAB+2.*s *CSCAB+2.*ss *SNCAB;
    QC   =((1.-Vfac)*dvc+(1.+Vfac)*uvc)*SNCAB+2.*sc*SNCAB+2.*ssc*CSCAB;
    QBAR =2*s;
    QBARC=0.0;
  }
  else if (POL<0)
  {
    Q    =(1.+Vfac)*dv + (1.- Vfac)*uv+2.*s;
    QC   =0.0;
    QBAR =2.*s *CSCAB + 2.*ss *SNCAB;
    QBARC=2.*sc*SNCAB + 2.*ssc*CSCAB;
  }
                                                                                
  // Use R1998 instead of Rworld, Jan. 2004 (UKYang)
  // IN RMODE, X->Xw?
  //double R = Rmodel(Xw,QSQ);
  //double RC = Rmodel(XPw,QSQ);
  double R,ErrR,RC,ErrRC;
  bool   GOODFIT=true;
  r1998(Xw, Q2,R, ErrR, GOODFIT);
  r1998(XPw,Q2,RC,ErrRC,GOODFIT);
                                                                                
  double F2   = Q + QBAR + QC + QBARC;
  double XF3  = Q - QBAR + QC - QBARC;
  double CPAR = (1.+pow(kTwoMP*X, 2)/Q2)/(1.+ R );
  double CPARC= (1.+pow(kTwoMP*XP,2)/Q2)/(1.+ RC);
  double X2F1 = CPAR*(Q+QBAR) + CPARC*(XP/X)*(QBARC + QC);
                                                                                
  double T1 = (1.-Y-0.25*kTwoMP*X*Y/ENU)*F2 + 0.5*X2F1*Y*Y;
  double T2 = Y*(1.-0.5*Y)*XF3;
  if (POL<0.) T2 = -T2;
  SIGMCQ = ( T1 + T2 );
                                                                                
  //
  // PROPAGATOR
  //
                                                                                
  double PROP=pow(1.0+Q2/kWmass2,2);
  SIGMCQ=SIGMCQ/PROP;
                                                                                
  return SIGMCQ;
}

