//
// GRV98 c++ implementation
//
// These are converted c++ code from original GRV98 FORTRAN codes
//
// Author : Inkyu Park
//
// University of Rochester
//
// e-mail) icpark@pas.rochester.edu
//
// 
//
//
//
#include <stdio.h>
#include <math.h>
double fint(double x,double y,int nx,double *ax,int ny,double *ay,double *Fxy)
{
  // determine cell
  int i,j;
  for (i=0;i<nx-1;i++) if (x>ax[i]&&x<ax[i+1]) break;
  for (j=0;j<ny-1;j++) if (y>ay[j]&&y<ay[j+1]) break;
  //
  double u=(x-ax[i])/(ax[i+1]-ax[i]); 
  double v=(y-ay[j])/(ay[j+1]-ay[j]); 
  //double g0=(1.0-u)*Fxy[i][j]+u*Fxy[i+1][j];
  //double g1=(1.0-u)*Fxy[i][j+1]+u*Fxy[i+1][j+1];
  // in c, 2-d array access isn't beautiful
  // I recommend you to use 1-d in the function
  double f00=Fxy[ny*i+j];
  double f01=Fxy[ny*i+j+1];
  double f10=Fxy[ny*(i+1)+j];
  double f11=Fxy[ny*(i+1)+j+1];
  double g0=(1.0-u)*f00 + u*f10;
  double g1=(1.0-u)*f01 + u*f11;
  //double g0=(1.0-u)*(double)*(Fxy+ny*i+j)  + u*(double)*(Fxy+ny*(i+1)+j)  ;
  //double g1=(1.0-u)*(double)*(Fxy+ny*i+j+1)+ u*(double)*(Fxy+ny*(i+1)+j+1);
  //double g0=(1.0-u)*(double)(Fxy[ny*i+j])  + u*(double)(Fxy[ny*(i+1)+j])  ;
  //double g1=(1.0-u)*(double)(Fxy[ny*i+j+1])+ u*(double)(Fxy[ny*(i+1)+j+1]);
  double fxy=(1.0-v)*g0+v*g1;
  //printf("cell (%d,%d):u=%f,v=%f,fxy=%f\n",i,j,u,v,fxy);
  return fxy;
}


int grv98pa(int iset,double X,double Q2,
	double &UV,double &DV,double &US,double &DS,double &SS,double &GL)
//-------------------------------------------------------------------
//                                                                  *
//  THE PARTON ROUTINE.                                             *
//                                    __                            *
//  INPUT:   ISET =  1 (LO),  2 (NLO, MS), or  3 (NLO, DIS)         *
//           X  =  Bjorken-x        (between  1.E-9 and 1.)         *
//           Q2 =  scale in GeV**2  (between  0.8 and 1.E6)         *
//                                                                  *
//  OUTPUT:  UV = u - u(bar),  DV = d - d(bar),  US = u(bar),       *
//           DS = d(bar),  SS = s = s(bar),  GL = gluon.            *
//           Always x times the distribution is returned.           *
//                                                                  *
//  COMMON:  The main program or the calling routine has to have    *
//           a common block  COMMON / INTINIP / IINIP , and the     *
//           integer variable  IINIP  has always to be zero when    *
//           GRV98PA is called for the first time or when  ISET     *
//           has been changed.                                      *
//                                                                  *
//  GRIDS:   1. grv98lo.grid, 2. grv98nlm.grid, 3. grv98nld.grid,   *
//           (1+1809 lines with 6 columns, 4 significant figures)   *
//                                                                  *
//-------------------------------------------------------------------
//                                                                  *
//    G R V  -  P R O T O N  - P A R A M E T R I Z A T I O N S      *
//                                                                  *
//                         1998 UPDATE                              *
//                                                                  *
//                 For a detailed explanation see                   *
//                  M. Glueck, E. Reya, A. Vogt :                   *
//       hep-ph/9806404  =  DO-TH 98/07  =  WUE-ITP-98-019          *
//                 (To appear in Eur. Phys. J. C)                   *
//                                                                  *
//  This package contains subroutines returning the light-parton    *
//  distributions in NLO (for the MSbar and DIS schemes) and LO;    * 
//  the respective light-parton, charm, and bottom contributions    *
//  to F2(electromagnetic); and the scale dependence of alpha_s.    *
//                                                                  *
//  The parton densities and F2 values are calculated from inter-   *
//  polation grids covering the regions                             *
//        Q^2/GeV^2  between   0.8   and  1.E6 ( 1.E4 for F2 )      *
//           x       between  1.E-9  and   1.                       *
//  Any call outside these regions stops the program execution.     *
//                                                                  *
//  At Q^2 = MZ^2, alpha_s reads  0.114 (0.125) in NLO (LO); the    *
//  heavy quark thresholds, QH^2 = mh^2, in the beta function are   *
//           mc = 1.4 GeV,  mb = 4.5 GeV,  mt = 175 GeV.            *
//  Note that the NLO alpha_s running is different from GRV(94).    * 
//                                                                  *
//   Questions, comments etc to:  avogt@physik.uni-wuerzburg.de     *
//                                                                  *
//-------------------------------------------------------------------
{
  static const int nX=68;
  static const int nQ=27;

  // The following arrays are for the interpolation

  static double XUVF[nX][nQ];
  static double XDVF[nX][nQ];
  static double XDEF[nX][nQ];
  static double XUDF[nX][nQ];
  static double XSF [nX][nQ];
  static double XGF [nX][nQ];
  static float  Parton[nX][nQ][6];

  static double aX[nX];
  static double aQ[nQ];
//..BJORKEN-X AND Q**2 VALUES OF THE GRID :
  static double XB[nX]=
  {   1e-9,1.8e-9,3.2e-9,5.7e-9,1.0e-8,1.8e-8,3.2e-8,5.7e-8,1.0e-7,1.8e-7,
    3.2e-7,5.7e-7,1.0e-6,1.4e-6,2.0e-6,3.0e-6,4.5e-6,6.7e-6,1.0e-5,1.4e-5,
      2e-5,3.0e-5,4.5e-5,6.7e-5,1.0e-4,1.4e-4,2.0e-4,3.0e-4,4.5e-4,6.7e-4,
      .001, .0014,  .002,  .003, .0045, .0067,   .01,  .014,   .02,   .03,
      .045, .0600, .0800, .1000, .1250, .1500, .1750, .2000, .2250, .2500,
      .275, .3000, .3250, .3500, .3750, .4000, .4500, .5000, .5500, .6000,
      .650, .7000, .7500, .8000, .8500, .9000, .9500,1.0000};
  static double QS[nQ]=
  {  0.8,  1.0,  1.3,  1.8,  2.7,  4.0,  6.4, 10.0, 16.0, 25.0,
     40.,  64., 100., 180., 320., 570.,1000.,1800.,3200.,5700.,
   1.0e4,2.2e4,4.6e4,1.0e5,2.2e5,4.6e5,1.0e6};
  static bool first=true;
  
  //
  // Initialization :
  //
  if (first)
  {
    printf("Reading data file...\n"); 
    FILE *file;
    if ((file=fopen("grv/grv98lo.data","r")))
    {
      for(int ix=0;ix<nX-1;ix++)
      {
        for(int iq=0;iq<nQ;iq++)
        {
          fscanf(file,"%f %f %f %f %f %f\n",
                   &Parton[ix][iq][0],
                   &Parton[ix][iq][1],
                   &Parton[ix][iq][2],
                   &Parton[ix][iq][3],
                   &Parton[ix][iq][4],
                   &Parton[ix][iq][5]);
        }
      }
      fclose(file);
    }
    else
    {
      printf("Error in opening GRV data file\n");
      return 1;
    }
    printf("done.%f\n",Parton[10][10][1]); 
    //
    //  ARRAYS FOR THE INTERPOLATION SUBROUTINE :
    //
    for (int iQ=0;iQ<nQ;iQ++)
    {
      for(int iX=0;iX<nX-1;iX++)
      {
        double XB0V = sqrt(XB[iX]);
        double XB0S = pow(XB[iX],-0.2);
        double XB1 = 1.0-XB[iX];
        double XB1_3=pow(XB1,3);
        double XB1_4=pow(XB1,4);
        double XB1_5=pow(XB1,5);
        double XB1_7=pow(XB1,7);
        XUVF[iX][iQ] = Parton[iX][iQ][0]/(XB1_3 * XB0V);
        XDVF[iX][iQ] = Parton[iX][iQ][1]/(XB1_4 * XB0V);
        XDEF[iX][iQ] = Parton[iX][iQ][2]/(XB1_7 * XB0V);
        XUDF[iX][iQ] = Parton[iX][iQ][3]/(XB1_7 * XB0S);
        XSF [iX][iQ] = Parton[iX][iQ][4]/(XB1_7 * XB0S);
        XGF [iX][iQ] = Parton[iX][iQ][5]/(XB1_5 * XB0S);
      }
      XUVF[nX-1][iQ] = 0.0;
      XDVF[nX-1][iQ] = 0.0;
      XDEF[nX-1][iQ] = 0.0;
      XUDF[nX-1][iQ] = 0.0;
      XSF [nX-1][iQ] = 0.0;
      XGF [nX-1][iQ] = 0.0;
    }
    for(int iX=0;iX<nX;iX++) aX[iX]=log10(XB[iX]);
    for(int iQ=0;iQ<nQ;iQ++) aQ[iQ]=log10(QS[iQ]);
    first = false;
  }

  //
  // Check of X and Q2 values  
  //
  if ((X<9e-9)||(X>1.0))
  {
    printf("PARTON INTERPOLATION: X OUT OF RANGE %f %f\n",X,Q2);
    return 1; 
  }
  if ((Q2<0.8)||(Q2>1e6))
  {
    printf("PARTON INTERPOLATION: Q2 OUT OF RANGE\n");
    return 1;
  }

  //
  // Interpolation
  //

  double logX=log10(X);
  double logQ=log10(Q2);
  double XV  =sqrt(X);
  double XS = 1.0/pow(X,-0.2);
  double X1=1.0-X;
  double X1_3=pow(X1,3);
  double X1_4=pow(X1,4);
  double X1_5=pow(X1,5);
  double X1_7=pow(X1,7);

  UV = fint(logX,logQ,nX,aX,nQ,aQ,(double*)XUVF)*X1_3 * XV;
  DV = fint(logX,logQ,nX,aX,nQ,aQ,(double*)XDVF)*X1_4 * XV;
  double DE = fint(logX,logQ,nX,aX,nQ,aQ,(double*)XDEF)*X1_7 * XV;
  double UD = fint(logX,logQ,nX,aX,nQ,aQ,(double*)XUDF)*X1_7 * XS;
  US = 0.5 * (UD - DE);
  DS = 0.5 * (UD + DE);
  SS = fint(logX,logQ,nX,aX,nQ,aQ,(double*)XSF) *X1_7 * XS;
  GL = fint(logX,logQ,nX,aX,nQ,aQ,(double*)XGF) *X1_5 * XS;
  return 0;
}

int grv98_lo_du(double X,double Q,
		double &uv,double &dv,double &us,double &ds,
		double &str,double &chm,double &btm,double &top,double &glu)
{
  chm = 0.0;
  btm = 0.0;
  top = 0.0;
  double Q2 = Q*Q;
  if(Q2<0.8) Q2=0.80;

  //printf("grv98_lo_du : uv=%f\n",uv);
  grv98pa(1,X,Q2,uv,dv,us,ds,str,glu);
  //printf("grv98_lo_du : %f %f %f %f %f %f \n",uv,dv,us,ds,str,glu);

  double deldu = -0.00817 + 0.0506*X + 0.0798*X*X;
  double float_cut = 0.00001; // Cut to avoid an infinity.
      
  double fac_val;
  if((uv+dv)>float_cut) fac_val = 1.0+ deldu*(uv/(uv+dv));
  else                  fac_val = 1.0+ 0.5*deldu; // At low x ==> uv=dv 

  double fac_sea;
  if((us+ds)>float_cut) fac_sea = 1.0+ deldu*(us/(us+ds));
  else                  fac_sea = 1.0+ 0.5*deldu; // At high x ==> usea = dsea

  uv = uv/fac_val*1.0;
  dv = (dv + uv*deldu)/fac_val*1.0;
       
  us = us/fac_sea*1.0;
  ds = (ds + us*deldu)/fac_sea*1.0;

  return 0;
}

double grv98_lo_f2(double X,double Q2,int IMODEL)
{
  double FS=sqrt(Q2);
  double Xw,f2el,pho_fac,val_fac,sea_fac,fpdf;
  if(IMODEL==1)
  {
    Xw = X;
    pho_fac = 1.0;
    val_fac = 1.0;
    sea_fac = 1.0;
    fpdf = 1.011;
  }
  else
  {
    Xw=X*(Q2+0.222)/(0.5*Q2*(1.0+sqrt(1.0+pow((1.876*X),2)/Q2))+0.4176*X);
    f2el = 1./pow((1.+Q2/0.71),4);
    pho_fac = (1. - f2el)     ;
    val_fac = (Q2+0.4852)/(Q2+0.6041);
    sea_fac = Q2/(Q2+0.3809);
    fpdf = 1.011;
  }
  double uv,dv,us,ds,str,chm,btm,top,glu;
  uv = 9.0;
  grv98_lo_du(Xw,FS,uv,dv,us,ds,str,chm,btm,top,glu);
  //printf("grv98_lo_f2 : %f %f %f %f %f %f %f %f %f\n",uv,dv,us,ds,str,chm,btm,top,glu);
  double F2val_LO = 4./9.*uv + 1./9.*dv;
  double F2sea_LO = 8./9.*us + 2./9*str + 2./9.*ds + 8./9.*chm;
  double F2=fpdf*(pho_fac*val_fac*F2val_LO + sea_fac*F2sea_LO);
 return F2;
}

double Rmodel(double X,double QSQ)
{
  const double C2= .125*.125;
  const double B1=.0635;
  const double B2=.5747;
  const double B3=-.3534;
  double SS=log10(QSQ/.04);
  double THETA = 1. + (12.*QSQ/(QSQ+1.)) * (C2/(C2 + X*X));
  return (B1/SS)*THETA + B2/QSQ + B3/(QSQ*QSQ+.09);
}                                                                                

double dr1998(double X,double Q2)
{
// Parameterizes the uncertainty in R1990 due to the statistical
// fluctuations in the data.  Values reflect an average of the R-values
// about a neighborhood of the specific (x,Q2) value.  That neighborhood
// is of size [+/-.05] in x, and [+/-33%] in Q2.  For details, see
// Reference.
//
// This subroutine is accurate over all (x,Q2), not only the SLAC deep
// inelastic range.  Where there is no data, for example in the resonance
// region, it returns a realistic uncertainty, extrapolated from the deep
// inelastic region (suitably enlarged).  We similarly estimate the
// uncertainty at very large Q2 by extrapolating from the highest Q2
// measurments.  For extremely large Q2, R is expected to fall to zero,
// so the uncertainty in R should not continue to grow.  For this reason
// DR1990 uses the value at 64 GeV for all larger Q2.
//
// XHIGH accounts for the rapidly diminishing statistical accuracy for
// x>.8, and does not contribute for smaller x.
  return .0078 -.013*X +(.070 -.39*X+.70*X*X)/(1.7+Q2);
}

void r1998(double X,double Q2,double &R,double &DR,bool GOODFIT)
//---------------------------------------------------------------
//X      : Bjorken x
//Q2     : Q squared in (GeV/c)**2
//R      :
//DR     : Absolute error on R
//GOODFIT:  = .TRUE. if the X,Q2 are within the range of the fit.
//------------------------------------------------------------------
//Model for R, based on a fit to world R measurements. Fit performed by
//program RFIT8 in pseudo-gaussian variable: log(1+.5R).  For details
//see Reference.
//
//Three models are used, each model has three free parameters.  The
//functional forms of the models are phenomenological and somewhat
//contrived.  Each model fits the data very well, and the average of
//the fits is returned.  The standard deviation of the fit values is
//used to estimate the systematic uncertainty due to model dependence.
//
// Statistical uncertainties due to fluctuations in measured values have
// have been studied extensively.  A parametrization of the statistical
// uncertainty of R1990 is presented in FUNCTION DR1990.
//
//
// Each model fits very well.  As each model has its own strong points
// and drawbacks, R1998 returns the average of the models.  The
// chisquare for each fit (237 points with 6 parameters) are:
//          ALL DATA  #PTS=237         |     X<  0.07 #PTS= 28
// FIT  #PARAM CHISQ  CHISQ/DF PROB(%) |  CHISQ  CHISQ/DF   PROB(%)
// R1998         217.4   0.94    73.1        28.7   1.06    37.5
// R1998a   6    219.4   0.95    69.8        28.8   1.07    37.1
// R1998b   6    217.7   0.94    72.6        27.8   1.03    42.2
// R1998c   6    221.9   0.96    65.5        30.9   1.15    27.4
//                                                                               
//
// This subroutine returns reasonable values for R for all x and for all
// Q2 greater than or equal to .3 GeV.
//
// The uncertainty in R originates in three sources:
//
//     D1 = uncertainty in R due to statistical fluctuations of the data
//          as reflected in the error of the fit.
//          It is parameterized in FUNCTION DR1990, for details see
//          Reference.
//
//    D2 = uncertainty in R due to possible model dependence, approxi-
//         mated by the variance between the models.
//
//     D3 = uncertainty in R due to possible epsilon dependent errors
//          in the radiative corrections, taken to be +/- .025.  See
//          theses (mine or Dasu's) for details.  This is copied from R1990
//                                                                               
//
// and the total error is returned by the program:
//
//     DR = is the total uncertainty in R, DR = sqrt(D1+D2+D3). 670
//          DR is my best estimate of how well we have measured R.  At
//          high Q2, where R is small, DR is typically larger than R.  If
//          you have faith in QCD, then, since R1990 = Rqcd at high Q2,
//          you might wish to assume DR = 0 at very high Q2.
//
// NOTE:    In many applications, for example the extraction of F2 from
//          measured cross section, you do not want the full error in R
//          given by DR.  Rather, you will want to use only the D1 and D2
//          contributions, and the D3 contribution from radiative
//          corrections propogates complexely into F2.  For more informa-
//          tion, see the documentation to dFRC in HELP.DOCUMENT, or
//          for explicite detail, see Reference.
//
//
//    ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
{
  double A[6]={4.8520E-02,5.4704E-01,2.0621E+00,-3.8036E-01,5.0896E-01,-2.8548E-02};
  double B[6]={4.8051E-02,6.1130E-01,-3.5081E-01,-4.6076E-01,7.1697E-01,-3.1726E-02};
  double C[6]={5.7654E-02,4.6441E-01,1.8288E+00,1.2371E+01,-4.3104E+01,4.1741E+01};

  double FAC = 1.+12.*Q2/(Q2+1.)*(pow(.125,2)/(pow(.125,2)+pow(X,2)));
  double RLOG=FAC/log10(Q2/.04);
                                                                                
// Model A
  double QP2=(A[1]/pow((pow(Q2,4)+pow(A[2],4)),.25))*(1.+A[3]*X +A[4]*pow(X,2));
// 1.07   .030
  double R_A=A[0]*RLOG +QP2*pow(X,A[5]);

// Model B
         QP2 = (1.+B[3]*X+B[4]*X*X)*(B[1]/Q2 +B[2]/(Q2*Q2+.09));
// 1.06   .042
  double R_B = B[0]*RLOG+QP2*pow(X,B[5]);

// Model C
  double Q2thr =C[3]*(X) +C[4]*X*X +C[5]*pow(X,3);
         QP2 =  C[1]/sqrt(pow((Q2-Q2thr),2)+pow(C[2],2));
  double R_C =  C[0]*RLOG+QP2;

  //
  //
  //

  R = (R_A+R_B+R_C)/3.;
  double D1 = dr1998(X,Q2);
  double D2 = sqrt((pow((R_A-R),2)+pow((R_B-R),2)+pow((R_C-R),2))/2.);
  double D3 = .023*(1.+.5*R);
  if (Q2<1.0 || X< .1) D3 = 1.5*D3;
  DR = sqrt(D1*D1+D2*D2+D3*D3);
  GOODFIT = true;
  if ((X<0.02)||(Q2<0.3)) GOODFIT = false;
  return;
}

void grv98f2(int ISET,double X,double Q2,double &F2C,double &F2B)
//*******************************************************************
//                                                                  *
//  THE F2 ROUTINE.                                                 *
//                                                                  *
//  INPUT :  ISET = 1 (LO),  2 (NLO).                               *
//           X  =  Bjorken-x        (between  1.E-9 and 1.)         *
//           Q2 =  scale in GeV**2  (between  0.8 and 1.E4)         *
//                                                                  *
//  OUTPUT:  F2L = F2(light), F2C = F2(charm), F2B = F2(bottom,)    *
//           F2  = sum, all given for electromagnetic proton DIS.   *
//                                                                  *
//  COMMON:  The main program or the calling routine has to have    *
//           a common block  COMMON / INTINIF / IINIF , and the     *
//           integer variable  IINIF  has always to be zero when    *
//           GRV98F2 is called for the first time or when  ISET     *
//           has been changed.                                      *
//                                                                  *
//  GRIDS:   1. grv98lof.grid, 2. grv98nlf.grid.                    *
//           (1+1407 lines with 3 columns, 4 significant figures)   *
//                                                                  *
//*****************************************************i*************
{
  static const int NSTRF=3;
  static const int NX=68;
  static const int NQ=21;
  static double aX[NX];
  static double aQ[NQ];
//  static double XF2LF[NX][NQ];
  static double XF2CF[NX][NQ];
  static double XF2BF[NX][NQ];
//
//..BJORKEN-X AND Q**2 VALUES OF THE GRID :
// 
  static double QS[NQ]={0.8E0, 
                 1.0E0, 1.3E0, 1.8E0, 2.7E0, 4.0E0, 6.4E0,
                 1.0E1, 1.6E1, 2.5E1, 4.0E1, 6.4E1,
                 1.0E2, 1.8E2, 3.2E2, 5.7E2,
                 1.0E3, 1.8E3, 3.2E3, 5.7E3,1.0E4};
       
  static double XB[NX]={1.0E-9, 1.8E-9, 3.2E-9, 5.7E-9,
                 1.0E-8, 1.8E-8, 3.2E-8, 5.7E-8,
                 1.0E-7, 1.8E-7, 3.2E-7, 5.7E-7,
                 1.0E-6, 1.4E-6, 2.0E-6, 3.0E-6, 4.5E-6, 6.7E-6,
                 1.0E-5, 1.4E-5, 2.0E-5, 3.0E-5, 4.5E-5, 6.7E-5,
                 1.0E-4, 1.4E-4, 2.0E-4, 3.0E-4, 4.5E-4, 6.7E-4,
                 1.0E-3, 1.4E-3, 2.0E-3, 3.0E-3, 4.5E-3, 6.7E-3,
                 1.0E-2, 1.4E-2, 2.0E-2, 3.0E-2, 4.5E-2, 0.06, 0.08,
                 0.1, 0.125, 0.15, 0.175, 0.2, 0.225, 0.25, 0.275,
                 0.3, 0.325, 0.35, 0.375, 0.4,  0.45, 0.5, 0.55,
                 0.6, 0.65,  0.7,  0.75,  0.8,  0.85, 0.9, 0.95, 1. };
//
//..CHECK OF X AND Q2 VALUES : 

  if ( (X<0.99e-9) ||   (X>   1.0 ) )
  {
    printf("STR.FCT. INTERPOLATION: X OUT OF RANGE\n");
  }
  if ( (Q2<0.799) ||(Q2>1.01e4))
  {
    printf("STR.FCT. INTERPOLATION: Q2 OUT OF RANGE\n");
  }
//
//..INITIALIZATION, IF REQUIRED :
//
//   SELECTION AND READING OF THE GRID : 
//   (COMMENT: FIRST NUMBER IN THE FIRST LINE OF THE GRID)
  static bool first=true;
  if (first)
  { 
    char *cfiles[2]={"grv/grv98lof.grid","grv/grv98nlf.grid"};
    float STRFCT[NSTRF][NQ][NX-1];
    FILE *file;
    if ((file=fopen(cfiles[ISET],"r")))
    {
      while (fgetc(file)!='\n');
      for(int M=0;M<NX-1;M++)
      for(int N=0;N<NQ;N++)
      {
        fscanf(file,"%f %f %f\n",
                  &STRFCT[0][N][M],&STRFCT[1][N][M],&STRFCT[2][N][M]);
      }
      fclose(file);
    }
    else
    {
      printf("NO OR INVALID STR.FCT. SET CHOICE\n");
    }

    //
    //...ARRAYS FOR THE INTERPOLATION SUBROUTINE :
    //
    for (int IQ=0;IQ<NQ;IQ++)
    {
      for (int IX=0;IX<NX-1;IX++)
      {
        double XBS = pow(XB[IX],0.2);
        double XB1 = 1.-XB[IX];
        XF2CF[IX][IQ] = STRFCT[1][IQ][IX]/pow(XB1,7)*XBS;
        XF2BF[IX][IQ] = STRFCT[2][IQ][IX]/pow(XB1,7)*XBS;
      }
      XF2CF[NX-1][IQ]=0.0;
      XF2BF[NX-1][IQ]=0.0;
    }
  
    for (int IX=0;IX<NX;IX++) aX[IX]=log10(XB[IX]);
    for (int IQ=0;IQ<NQ;IQ++) aQ[IQ]=log10(QS[IQ]);
    first=false;
  }
  //
  //..CONTINUATION, IF INITIALIZATION WAS DONE PREVIOUSLY.
  //
  //
  //..INTERPOLATION :
  double logX=log10(X);
  double logQ=log10(Q2);
  double X1=1.- X;
  double XS=pow(X,-0.2);
  F2C=fint(logX,logQ,NX,aX,NQ,aQ,(double*)XF2CF)*pow(X1,7)*XS;
  F2B=fint(logX,logQ,NX,aX,NQ,aQ,(double*)XF2BF)*pow(X1,7)*XS;
  return;
}


/*
int main()
{
  grv98_lo_f2(0.55,110.0,1);
  grv98_lo_f2(0.55,110.0,1);
}
*/
